# This script measures pitch, intensity and formants every 10ms
# Based on the code written by Setsuko Shirai,
# a little modified by Jangwon Kim
# Contact ssetsuko@u.washington.edu
# Contact jangwon@usc.edu
# For EMA database
# Change "PATH"

# Parameter -----------------------
type_file$ = "wav"
# below is only for 'To LPC + To Pitch'
# prediction_order = 11

# Pitch (ac) parameter
# male: 75-300, female: 100-600 in general.
minimum_pitch = 75
maximum_pitch = 500
max_num_candidate = 15
sil_thld = 0.06
vo_thld = 0.3
octv_cost = 0.05
octv_jump_cost = 0.35
v_uv_cost = 0.18

win_length = 0.025
win_shift = 0.005

form supply_arguments
	sentence input_directory  "PATH"\EmaLabelledWavFile
	sentence output_directory "PATH"\EmaTF0EFsBws_carefullyExtractedFeatures_allsnt
endform

# finding files we are looking for
Create Strings as file list... list 'input_directory$'\*.'type_file$'
# the name of files - later we could track each file
numberOfFiles = Get number of strings
for ifile to numberOfFiles
	select Strings list
	fileName$ = Get string... ifile
	Read from file... 'input_directory$'\'fileName$'
endfor

select all
numSelected = numberOfSelected ("Sound")

# change the name of each file - for batch processing
for i to numSelected
	select all
	currSoundID = selected ("Sound", i)
	select 'currSoundID'
	currName$ = "word_'i'"
	Rename... 'currName$'
endfor

for i to numSelected 
	select Sound word_'i'

	# get the finishing time of the Sound file
	fTime = Get finishing time
	fTime = floor(fTime * 100) / 100

      	# Use numTimes in the loop
	numTimes = fTime / win_shift
	newName$ = "word_'i'"
	select Sound word_'i'

      	# 1st argument: New sample rate 2nd argument: Precision (samples)
	# Resample... 'new_sample_rate' 50 apply this only when you use To LPC for formant extraction

	# 1st argument: Time step (s), 2nd argument: Minimum pitch for Analysis, 
	# 3rd argument: Maximum pitch for Analysis
	# Simple pitch extraction
	# To Pitch... 'win_shift' 'minimum_pitch' 'maximum_pitch'
	# AC pitch extraction: arguments are below: 
	# time_step min_f0 max_#candidate, Very accurate, Silence Threshold, Voicing Threshold
	# Octave cost, Octave-jump cost, Voiced/unvoiced cost, max_f0
	To Pitch (ac)... 'win_shift' 'minimum_pitch' 'max_num_candidate' no 'sil_thld' 'vo_thld' 'octv_cost' 'octv_jump_cost' 'v_uv_cost' 'maximum_pitch'

	Rename... 'newName$'

	# select Sound word_'i'_'new_sample_rate'
	select Sound word_'i'
	To Intensity... 'minimum_pitch' 0
	
	Create Table... table_word_'i' numTimes 3
	Set column label (index)... 1 T
	Set column label (index)... 2 F0
	Set column label (index)... 3 E

	for itime to numTimes
		select Pitch word_'i'
		curtime = win_shift * itime

		f0 = 0
		f0 = Get value at time... 'curtime' Hertz Linear
		f0$ = fixed$ (f0, 2)

		if f0$ = "--undefined--"
			f0$ = "0"
		endif

		curtime$ = fixed$ (curtime, 5)
		 
		# select Intensity word_'i'_'new_sample_rate'
		select Intensity word_'i'
		intensity = Get value at time... 'curtime' Cubic
		
		
		intensity$ = fixed$ (intensity, 2)
		if intensity$ = "--undefined--"
			intensity$ = "0"
		endif

		select Table table_word_'i'
		Set numeric value... itime T 'curtime$'
		Set numeric value... itime F0 'f0$' 
		Set numeric value... itime E  'intensity$'
	endfor
	select Strings list
	fileName$ = Get string... i
	select Table table_word_'i'
	Write to table file... 'output_directory$'\'fileName$'.txt
endfor
