# This python script performs frame-level HNR extraction using Praat
# It assumes that praat is installed in your machine
# USE: python ext_HNR.py dir_wav dir_out time_step min_f0 sil_thld periods_per_win
# eg)  python ext_HNR.py ./dum_wav ./dum_out 0.01 50 0.1 1.0
# This script is written by Jangwon Kim
# University of Southern California
# email to jangwon@usc.edu for any question

import os
import sys

dir_wav = sys.argv[1]
dir_out = sys.argv[2]
time_step = sys.argv[3]
min_f0 = sys.argv[4]
sil_thld = sys.argv[5]
periods_per_win = sys.argv[6]

os.system(' '.join(('mkdir -p',dir_out)))

# create wavfile list
cmd_s=''.join(('ls ',dir_wav,' > tmp_list'))
os.system(cmd_s)
# load the wav file list
f = open('tmp_list')
lines = f.readlines()
f.close()
os.system('rm tmp_list')

# extract HNR feature file for each wav file
for line in lines:
  bname = line.rstrip('\r\n')

  # paths of input and output files
  fpath_in = ''.join((dir_wav,'/',bname))
  fpath_out = ''.join((dir_out,'/',bname.replace('.wav','.HNR')))

  # create temporary praat script for the current file
  fpath_tmp_script = 'tmp_script'
  f = open(fpath_tmp_script, 'w')
  f.write('infile$ = \"' + fpath_in + '\"\n')
  f.write('outfile$ = \"' + fpath_out + '\"\n')
  f.write('Read from file... \'infile$\'\n')
  f.write('object_name$ = selected$(\"Sound\")\n')
  f.write(' '.join(('To Harmonicity (cc)...',time_step,min_f0,sil_thld,periods_per_win,'\n')))
  f.write('harmonicity = selected (\"Harmonicity\", 1)\n')
  f.write('To Matrix\n')
  f.write('Save as headerless spreadsheet file... ' + fpath_out + '\n')
  f.write('select Matrix \'object_name$\'\n')
  f.write('plus Harmonicity \'object_name$\'\n')
  f.write('plus Sound \'object_name$\'\n')
  f.write('Remove\n')
  f.close()

  # feature extraction
  cmd_s = ' '.join(('praat','tmp_script'))
  print cmd_s
  os.system(cmd_s)

os.system('rm tmp_script')
